#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé le Apr 19 19:47:48 2022

@auteur: david ALBERTO (www.astrolabe-science.fr)

Documentation Skyfield : https://rhodesmill.org/skyfield/
"""
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.dates as mdates#format des graduations de dates
from skyfield.api import load, utc
from skyfield.framelib import ecliptic_frame
import datetime as dt  # créaction de dates
# from datetime import datetime, timedelta
import locale# noms des mois en français, sur les graduations

# Réglages utilisateur :------------------------
locale.setlocale(locale.LC_ALL, 'fr_FR.UTF-8')#dates en français
annee=2025
couleurFond='#c7cef2'
#Liste des planètes et de leurs couleurs :

listenoms = ['Mercure','Vénus','Mars','Jupiter','Saturne', 'Lune'
# 'Uranus', 'Neptune'
]

listecouleurs = ['slategray','slateblue','orangered','saddlebrown',
'darkgoldenrod', 'gray'
 # 'Uranus' : 'blue',
# 'Neptune' : 'darkblue'
]

plt.rcParams["font.family"] = "Roboto"# ou autre police installée
#------------------------------------------------------------------------------

# (Télé)chargement du fichier d'ephémérides de421.bsp (16 MB, période 1849-2050)
eph = load('../skyfield-ephemerides/de421.bsp')# si le fichier est déjà chargé sur le disque (chemin à adapter).
# eph=api.load('de421.bsp')# pour télécharger automatiquement le fichier, dans le dossier du fichier Python

soleil = eph['sun']
mercure = eph['mercury']
venus = eph['venus']
terre= eph['earth']
mars = eph['mars']
jupiter = eph['jupiter barycenter']
saturne = eph['saturn barycenter']
# uranus = eph['uranus barycenter']
# neptune = eph['neptune barycenter']
lune = eph['moon']

listeplanetes = [mercure, venus, mars, jupiter, saturne, lune
# uranus, neptune
]

# Fonction renvoyant l'élongation d'une planète à une date donnée :
def elongation_at(t,planete):
    #calcul des longitudes apparentes du Soleil et de la planète, depuis la Terre
    e = terre.at(t)
    _, slon, _ = e.observe(soleil).apparent().frame_latlon(ecliptic_frame)
    _, plon, _ = e.observe(planete).apparent().frame_latlon(ecliptic_frame)
    # booléen déterminant si la planète est en élongation Est :
    is_east = (plon.degrees - slon.degrees) % 360.0 < 180.0
    s = e.observe(soleil).apparent()
    p = e.observe(planete).apparent()
    if is_east :
        elong = -s.separation_from(p).degrees
    else : elong = s.separation_from(p).degrees
    return elong

# vectorisation de la fonction, pour qu'elle puisse recevoir une liste de dates, comme 1er argument.
elongationvect = np.vectorize(elongation_at)
#------------------------------------------------------------------------------
"""
Remarque sur les échelles de dates :
J'utilise ici 2 échelles de dates :
    - échelle 'Skyfield' pour les calculs d'élongation
    - échelle 'datetime' pour créer la liste 'époque' (année complète) et aussi
    pour régler l'axe vertical du graphique
Une conversion est nécessaire pour passer d'une échelle à l'autre.
"""
ts = load.timescale()#crée une échelle de dates Skyfield
date_debut = dt.datetime(annee, 1, 1, tzinfo=utc)
date_fin = dt.datetime(annee, 12, 31, tzinfo=utc)


#Création d'une liste de dates, appelée "epoque" :
epoque=[]
pas=dt.timedelta(days=2)#pas de temps, en jours
date=date_debut
while date<date_fin:
    epoque.append(date)
    date=date+pas

#------------------------------------------------------------------------------
#création de la figure et du graphique :
fig, ax=plt.subplots(tight_layout=True, facecolor=couleurFond)
ax.set_title("Élongations des planètes en %i"%(annee), fontsize=18)
ax.invert_yaxis()#inversion du sens de l'axe y (dates descendantes)
ax.set_xlabel("Élongation (degrés)")
fig.set_size_inches(21/2.54, 29.7/2.54)#dimensions de la figure
ax.set_ylim(date_fin, date_debut)
ax.set_xlim(-180, 180)
ax.set_xticks(np.arange(-180, 180+20, 20))

#Mise en forme des graduations de dates en y :
locator1 = mdates.AutoDateLocator()
formatter = mdates.ConciseDateFormatter(locator1)
jours=mdates.DayLocator(bymonthday=(10, 20))#graduation secondaire tous les 10 jours
ax.yaxis.set_major_locator(locator1)
ax.yaxis.set_minor_locator(jours)
ax.yaxis.set_major_formatter(formatter)
ax.yaxis.set_minor_formatter(formatter)
#------------------------------------------------------------------------------

dates = ts.utc(epoque) # conversion des dates en format skyfield

for i, nom in enumerate(listenoms):
    idx = listenoms.index(nom)
    p = listeplanetes[idx]
    couleur = listecouleurs[idx]
    x = elongationvect(dates, p)
    if nom == 'Lune':
        point = 20
        lw=0.4
    else:
        point=8
        lw=0
    ax.scatter(x, epoque, s = point, c = couleur, label = nom,
               lw=lw, ec='k')

ax.axvline(0, c='darkorange', lw = 5) # ligne verticale (direction du Soleil)

fig.text(0.25, 0.015, "SOIR",  ha='center')
fig.text(0.75, 0.015, "MATIN",  ha='center')

# --------------------------------------------

#légende
legendes=plt.legend(frameon=False,# suppression du cadre de la légende
                    labelspacing=1.5,# espacer les légendes
                    markerscale=0,# grossir les points
                    )
for i, texte in enumerate(legendes.get_texts()):
    texte.set(color='white',
              bbox=dict(fc = listecouleurs[i])# encadrer les légendes, avec les couleurs des planètes
              )

ax.set_axisbelow(True)#place la grille sous les points tracés.
ax.grid(which='minor', alpha=0.5)# grille secondaire estompée
ax.grid(which='major', color='gray')

fig.text(0.05,0.015,"D. Alberto (www.astrolabe-science.fr)",fontsize=6)

NomFichier="ElongationPlanetes"+str(annee)
fig.savefig(NomFichier+".png", dpi=200)
fig.savefig(NomFichier+".pdf")
